<?php

/*
  Plugin Name: UI for WP Simple Paypal Shopping Cart
  Plugin URI: http://orbisius.com/site/products/wordpress-simple-paypal-shopping-cart-ui/
  Description: Adds a button in the rich text editor that will generate the short code for the WordPress Simple Paypal Shopping Cart plugin. To support the plugin development please <a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=7APYDVPBCSY9A" target="_blank">Donate</a>
  Tags: WordPress shopping cart, Paypal shopping cart, Paypal shopping cart UI,stre,e-store,store, online shop, shopping cart, wordperss ecommerce, sell digital products,wp,wp store
  Version: 1.0.5
  Author: Svetoslav Marinov (Slavi) | http://webweb.ca
  Author URI: http://WebWeb.ca
  License: GPL v2
 */

/*
  Copyright 2011-2020 Svetoslav Marinov (slavi@slavi.biz)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

define('WEBWEB_WP_UI_WPSPPSC_VERSION', '1.0.0');
define('WEBWEB_WP_UI_WPSPPSC_BASE_DIR', dirname(__FILE__)); // e.g. // htdocs/wordpress/wp-content/plugins/wp-command-center/
define('WEBWEB_WP_UI_WPSPPSC_DIR_NAME', basename(WEBWEB_WP_UI_WPSPPSC_BASE_DIR)); // e.g. wp-command-center

// Make sure we don't expose any info if called directly
if (!function_exists('add_action')) {
    echo "Hi there!  I'm just a plugin, not much I can do when called directly.";
    exit;
}

$webweb_ms_obj = new WebWeb_UI_WPSimplePaypalShoppingCart();
add_action('init', array($webweb_ms_obj, 'init'));

class WebWeb_UI_WPSimplePaypalShoppingCart {
    private $plugin_tinymce_name = 'wwwpuiwpsppsc'; // if you change it update the tinymce/editor.js;
    private $plugin_id_str = 'wp-simple-paypal-shopping-cart-ui'; //

    private $site_url = null; // filled in later
    private $plugin_url = null; // filled in later
    private $plugin_dir_name = null; // filled in later
    private $plugin_data_dir = null; // plugin data directory. for reports and data storing. filled in later
    private $plugin_name = 'UI for WordPress Simple Paypal Shopping Cart'; //
    private $plugin_business_sandbox = false; // sandbox or live ???
    private $plugin_business_email_sandbox = 'seller_1264288169_biz@slavi.biz'; // used for paypal payments
    private $plugin_business_email = 'billing@WebWeb.ca'; // used for paypal payments
    private $plugin_business_ipn = 'http://webweb.ca/wp/hosted/payment/ipn.php'; // used for paypal IPN payments
    //private $plugin_business_status_url = 'http://localhost/wp/hosted/payment/status.php'; // used after paypal TXN to to avoid warning of non-ssl return urls
    private $plugin_business_status_url = 'https://secure.webweb.ca/webweb.ca/wp/hosted/payment/status.php'; // used after paypal TXN to to avoid warning of non-ssl return urls
    private $plugin_support_email = 'help@WebWeb.ca'; //
    private $plugin_support_link = 'http://miniads.ca/widgets/contact/profile/like-gate?height=200&width=500&description=Please enter your enquiry below.'; //
    private $plugin_admin_url_prefix = null; // filled in later
    private $plugin_home_page = 'http://webweb.ca/site/products/like-gate/';

    function __construct() {
        $site_url = get_settings('siteurl');

        add_action('plugins_loaded', array($this, 'init'), 100);
        
        $site_url = get_settings('siteurl');

        $this->site_url = $site_url;
        $this->plugin_url = get_settings('siteurl') . '/wp-content/plugins/' . WEBWEB_WP_UI_WPSPPSC_DIR_NAME . '/';
        $this->plugin_dir_name = basename(dirname(__FILE__)); // e.g. wp-command-center; this can change e.g. a 123 can be appended if such folder exist
        $this->plugin_data_dir = dirname(__FILE__) . '/data';
        $this->plugin_url = $site_url . '/wp-content/plugins/' . $this->plugin_dir_name . '/';
        $this->plugin_support_link .= '&css_file=' . urlencode(get_bloginfo('stylesheet_url'));
        $this->plugin_admin_url_prefix = $site_url . '/wp-admin/admin.php?page=' . $this->plugin_dir_name;
        $this->plugin_admin_settings_url = $site_url . '/wp-admin/options-general.php?page=' . $this->plugin_dir_name . '/wordpress-simple-paypal-shopping-cart-ui.php';
        $this->plugin_settings_key = $this->plugin_id_str . '_settings';
    }

    /**
     * Allows access to some private vars
     * @param str $var
     */
    public function get($var) {
        if (isset($this->$var) /* && (strpos($var, 'plugin') !== false) */) {
            return $this->$var;
        }
    }
    
    /**
     * handles the init
     */
    function init() {
        wp_enqueue_script('jquery');
        
        if (is_admin()) {
            // Administration menus
            add_action('admin_menu', array($this, 'administration_menu'));
            add_action('admin_init', array($this, 'add_buttons'));
            add_action('admin_init', array($this, 'register_settings'));
            add_action('admin_notices', array($this, 'notices'));            
        } else {
            // Do nothing on public side
        }
    }

    /**
     * Sets the setting variables
     */
    function register_settings() { // whitelist options
        register_setting($this->plugin_dir_name, $this->plugin_settings_key);
    }
    
    /**
     * Checks if WP simpple shopping cart is installed.
     */
    function notices() {
        if (!empty($_REQUEST['dismiss_warning'])) {
            return;
        }
        
        $simple_cart_path = dirname(WEBWEB_WP_UI_WPSPPSC_BASE_DIR) . '/wordpress-simple-paypal-shopping-cart';

        $dismiss = <<<DISMISS_EOF
<p><a href="{$this->plugin_admin_settings_url}&dismiss_warning=1">Dismiss (I have installed WP Paypal Shopping Cart and it's working just fine)</a>.</p>
DISMISS_EOF;
        
        if (!is_file($simple_cart_path . '/wp_shopping_cart.php')) {
            $opts = get_option($this->plugin_settings_key);

            // the users hasn't dismissed the message yet.
            if (empty($opts['dismiss_warning'])) {
                echo "<div id='wwwpuiwpsppsc-warning' class='error fade'><p><strong>"
                    . sprintf(__('WP Paypal Shopping UI: WP Paypal Shopping Cart Plugin doesn\'t seem to be installed/activated.' . "</strong>")
                    . __('<br/><br/>Download it from <a href="http://www.tipsandtricks-hq.com/wordpress-simple-paypal-shopping-cart-plugin-768" target="_blank">http://www.tipsandtricks-hq.com/wordpress-simple-paypal-shopping-cart-plugin-768</a>.')
                    , "plugins.php?page=wwwpuiwpsppsc-key-config") . "</p>$dismiss</div>";
            }
        }
    }

    /**
     * Adds the settings in the admin menu
     */
    public function administration_menu() {
        //$main_page = '/menu/dashboard.php';
        //add_menu_page(__('Member Status', 'webweb_member_status'), __('Member Status', 'webweb_member_status'), 'manage_options', WEBWEB_WP_UI_WPSPPSC_DIR_NAME . '/menu/dashboard.php', null, null);

        add_options_page(__("UI for WP Paypal Shopping Cart", "WPSPPSC"), __("UI for WP Paypal Shopping Cart", "WPSPPSC"), 'manage_options', __FILE__, array($this, 'options'));

        // when plugins are show add a settings link near my plugin for a quick access to the settings page.
        add_filter('plugin_action_links', array($this, 'add_plugin_settings_link'), 10, 2);
    }

    /**
     * Outputs some options info. No save for now.
     */
    function options() {
        $buffer = '';
        
        if (!empty($_REQUEST['dismiss_warning'])) {
            $opts['dismiss_warning'] = 1;
            update_option($this->plugin_settings_key, $opts);
            
            // can't redirect headers are already sent

            $buffer = <<<EOF
            <p>The notice has been dismissed.</p>
            <p>Please <a href="{$this->plugin_admin_settings_url}">Continue</a></p>
EOF;
        } else {
            $support_url = "http://miniads.ca/widgets/contact/profile/{$this->plugin_id_str}?font=Arial,Sans-Serif&font_size=12&height=200&width=500&description=Please enter your enquiry below."
               . '&css_file=' . urlencode(get_bloginfo('stylesheet_url'));
                    
            $buffer = '<div class="wrap">
                            <h2>' . (__("UI for WordPress Simple Paypal Shopping Cart Options", "WSPSC")) . '</h2>';
            $buffer .= <<<OPTIONS_EOF
    <div id="poststuff">
        <div id="post-body">
      <p style="border:2px dashed red; padding:5px; __float:right;">
            Free offer from the Developer:
            <a href="http://orbisius.com/go/intro2site?s=ui-for-simple-shopping-cart"
                                                target="_blank">Free e-book: How to Build a Website Using WordPress: Beginners Guide</a>
      </p>

            The plugin adds a new button in the RichText editor that way you can quickly add products for
                    the <a href="http://www.tipsandtricks-hq.com/wordpress-simple-paypal-shopping-cart-plugin-768" target="_blank">WP Simple Paypal Shopping Cart</a> (plugin created by TipsAndTricks).
            <br/>
            The UI plugin doesn't require any configuration at the moment however you'll have to configure WP Simple Paypal Shopping Cart.

            <h2>Usage</h2>

            When you create a new or edit page/post and click on <img src="{$this->plugin_url}tinymce/icon.png" alt="icon"/>.

            You should see the following new icon (if you use the rich text editor)

            <p><img src="{$this->plugin_url}screenshot-1.png" alt="new icon" width="500"/></p>

            <p>
            <a href="{$this->plugin_url}screenshot-2.png" target="_blank" title="Click for a bigger image"><img src="{$this->plugin_url}screenshot-2.png" alt="UI" width="250" /></a>
            </p>

            {$this->generate_newsletter_box()}
            
            {$this->generate_donate_box()}
            
            {$this->generate_contact_box()}

            <h2>About</h2>

            <p>This plugin was created by Svetoslav Marinov (Slavi) - <a href="http://slavi.biz" target="_blank">slavi.biz</a> |  <a href="http://WebWeb.ca" target="_blank">WebWeb.ca</a> </p>
        </div> <!-- /post-body -->
    </div> <!-- /poststuff -->
</div> <!-- /wrap -->
<link rel='stylesheet' href='{$this->plugin_url}/css/main.css' type='text/css' media='all' />
OPTIONS_EOF;
        }

        echo $buffer;
        
        //return $buffer;
    }

    // Add the ? settings link in Plugins page very good
    function add_plugin_settings_link($links, $file) {
        if ($file == plugin_basename(__FILE__)) {
            $settings_link = '<a href="options-general.php?page=' . dirname(plugin_basename(__FILE__)) . '/' . basename(__FILE__) . '">' . (__("Settings", "WPSPPSC")) . '</a>';
            array_unshift($links, $settings_link);
        }

        return $links;
    }

    /**
     * Adds buttons only for RichText mode
     * @return void
     */
    function add_buttons() {
        // Don't bother doing this stuff if the current user lacks permissions
        if (!current_user_can('edit_posts') && !current_user_can('edit_pages')) {
            return;
        }

        // Add only in Rich Editor mode
        if (get_user_option('rich_editing') == 'true') {
            // add the button for wp2.5 in a new way
            add_filter("mce_external_plugins", array(&$this, "add_tinymce_plugin"), 5);
            add_filter('mce_buttons', array(&$this, 'register_button'), 5);
        }
    }

    // used to insert button in wordpress 2.5x editor
    function register_button($buttons) {
        array_push($buttons, "separator", $this->plugin_tinymce_name);

        return $buttons;
    }

    // Load the TinyMCE plugin : editor_plugin.js (wp2.5)
    function add_tinymce_plugin($plugin_array) {
        $plugin_array[$this->plugin_tinymce_name] = $this->plugin_url . 'tinymce/editor_plugin.min.js';

        return $plugin_array;
    }

    /**
     * Allows access to some private vars
     * @param str $var
     */
    public function generate_newsletter_box() {
        $file = WEBWEB_WP_UI_WPSPPSC_BASE_DIR . '/zzz_newsletter_box.html';

        $buffer = WebWeb_UI_WPSimplePaypalShoppingCartUtil::read($file);

        wp_get_current_user();
        global $current_user;
        $user_email = $current_user->user_email;

        $replace_vars = array(
            '%%PLUGIN_URL%%' => $this->get('plugin_url'),
            '%%USER_EMAIL%%' => $user_email,
        );

        $buffer = str_replace(array_keys($replace_vars), array_values($replace_vars), $buffer);

        return $buffer;
    }

    /**
     * Loads contact tpl. It needs access to $webweb_ms_obj (global var)
     * @param void
     */
    public function generate_contact_box() {
        $webweb_ms_obj = $this;
        $file = WEBWEB_WP_UI_WPSPPSC_BASE_DIR . '/zzz_contact_form.php';

        ob_start();
        include($file);
        $result = ob_get_contents();
        ob_end_clean();

        return $result;
    }

    /**
     * Allows access to some private vars
     * @param str $var
     */
    public function generate_donate_box() {
        $msg = '';
        $file = WEBWEB_WP_UI_WPSPPSC_BASE_DIR . '/zzz_donate_box.html';

        if (!empty($_REQUEST['error'])) {
            $msg = $this->message('There was a problem with the payment.');
        }

        if (!empty($_REQUEST['ok'])) {
            $msg = $this->message('Thank you so much!', 1);
        }

        $return_url = WebWeb_UI_WPSimplePaypalShoppingCartUtil::add_url_params($this->get('plugin_business_status_url'), array(
            'r' => $this->get('plugin_admin_settings_url') . '&ok=1', // paypal de/escapes
            'status' => 1,
        ));

        $cancel_url = WebWeb_UI_WPSimplePaypalShoppingCartUtil::add_url_params($this->get('plugin_business_status_url'), array(
            'r' => $this->get('plugin_admin_settings_url') . '&error=1', //
            'status' => 0,
        ));

        $replace_vars = array(
            '%%MSG%%' => $msg,
            '%%AMOUNT%%' => '9.99',
            '%%BUSINESS_EMAIL%%' => $this->plugin_business_email,
            '%%ITEM_NAME%%' => $this->plugin_name . ' Donation',
            '%%ITEM_NAME_REGULARLY%%' => $this->plugin_name . ' Donation (regularly)',
            '%%PLUGIN_URL%%' => $this->get('plugin_url'),
            '%%CUSTOM%%' => http_build_query(array('site_url' => $this->site_url, 'product_name' => $this->plugin_id_str, 'category' => 'wp')),
            '%%NOTIFY_URL%%' => $this->get('plugin_business_ipn'),
            '%%RETURN_URL%%' => $return_url,
            '%%CANCEL_URL%%' => $cancel_url,
        );

        // Let's switch the Sandbox settings.
        if ($this->plugin_business_sandbox) {
            $replace_vars['paypal.com'] = 'sandbox.paypal.com';
            $replace_vars['%%BUSINESS_EMAIL%%'] = $this->plugin_business_email_sandbox;
        }

        $buffer = WebWeb_UI_WPSimplePaypalShoppingCartUtil::read($file);
        $buffer = str_replace(array_keys($replace_vars), array_values($replace_vars), $buffer);

        return $buffer;
    }

    /**
     * Outputs a message (adds some paragraphs)
     */
    function message($msg, $status = 0) {
        $id = $this->plugin_id_str;
        $cls = empty($status) ? 'app_error fade' : 'app_success';

        $str = <<<MSG_EOF
<div id='$id-notice' class='app_message_box $cls'><p><strong>$msg</strong></p></div>
MSG_EOF;
        return $str;
    }

    /**
     * a simple status message, no formatting except color
     */
    function msg($msg, $status = 0) {
        $id = $this->plugin_id_str;
        $cls = empty($status) ? 'app_error' : 'app_success';

        $str = <<<MSG_EOF
<div id='$id-notice' class='$cls'><strong>$msg</strong></div>
MSG_EOF;
        return $str;
    }

    /**
     * a simple status message, no formatting except color, simpler than its brothers
     */
    function m($msg, $status = 0) {
        $cls = empty($status) ? 'app_error' : 'app_success';

        $str = <<<MSG_EOF
<span class='$cls'>$msg</span>
MSG_EOF;
        return $str;
    }

}

class WebWeb_UI_WPSimplePaypalShoppingCartUtil {
    // options for read/write methods.
    const FILE_APPEND = 1;
    const UNSERIALIZE_DATA = 2;
    const SERIALIZE_DATA = 3;

    /**
     * Gets the content from the body, removes the comments, scripts
     * Credits: http://php.net/manual/en/function.strip-tags.phpm /  http://networking.ringofsaturn.com/Web/removetags.php
     * @param string $buffer
     * @string string $buffer
     */
    public static function html2text($buffer = '') {
        // we care only about the body so it must be beautiful.
        $buffer = preg_replace('#.*<body[^>]*>(.*?)</body>.*#si', '\\1', $buffer);
        $buffer = preg_replace('#<script[^>]*>.*?</script>#si', '', $buffer);
        $buffer = preg_replace('#<style[^>]*>.*?</style>#siU', '', $buffer);
//        $buffer = preg_replace('@<style[^>]*>.*?</style>@siU', '', $buffer); // Strip style tags properly
        $buffer = preg_replace('#<[a-zA-Z\/][^>]*>#si', ' ', $buffer); // Strip out HTML tags  OR '@<[\/\!]*?[^<>]*\>@si',
        $buffer = preg_replace('@<![\s\S]*?--[ \t\n\r]*>@', '', $buffer); // Strip multi-line comments including CDATA
        $buffer = preg_replace('#[\t\ ]+#si', ' ', $buffer); // replace just one space
        $buffer = preg_replace('#[\n\r]+#si', "\n", $buffer); // replace just one space
        //$buffer = preg_replace('#(\s)+#si', '\\1', $buffer); // replace just one space
        $buffer = preg_replace('#^\s*|\s*$#si', '', $buffer);

        return $buffer;
    }

    /**
     * Gets the content from the body, removes the comments, scripts
     *
     * @param string $buffer
     * @param array $keywords
     * @return array - for now it returns hits; there could be some more complicated results in the future so it's better as an array
     */
    public static function match($buffer = '', $keywords = array()) {
        $status_arr['hits'] = 0;

        foreach ($keywords as $keyword) {
            $cnt = preg_match('#\b' . preg_quote($keyword) . '\b#si', $buffer);

            if ($cnt) {
                $status_arr['hits']++; // total hits
                $status_arr['matches'][$keyword] = array('keyword' => $keyword, 'hits' => $cnt,); // kwd hits
            }
        }

        return $status_arr;
    }

    /**
     * @desc write function using flock
     *
     * @param string $vars
     * @param string $buffer
     * @param int $append
     * @return bool
     */
    public static function write($file, $buffer = '', $option = null) {
        $buff = false;
        $tries = 0;
        $handle = '';

        $write_mod = 'wb';

        if ($option == self::SERIALIZE_DATA) {
            $buffer = serialize($buffer);
        } elseif ($option == self::FILE_APPEND) {
            $write_mod = 'ab';
        }

        if (($handle = @fopen($file, $write_mod))
                && flock($handle, LOCK_EX)) {
            // lock obtained
            if (fwrite($handle, $buffer) !== false) {
                @fclose($handle);
                return true;
            }
        }

        return false;
    }

    /**
     * @desc read function using flock
     *
     * @param string $vars
     * @param string $buffer
     * @param int $option whether to unserialize the data
     * @return mixed : string/data struct
     */
    public static function read($file, $option = null) {
        $buff = false;
        $read_mod = "rb";
        $tries = 0;
        $handle = false;

        if (($handle = @fopen($file, $read_mod))
                && (flock($handle, LOCK_EX))) { //  | LOCK_NB - let's block; we want everything saved
            $buff = @fread($handle, filesize($file));
            @fclose($handle);
        }

        if ($option == self::UNSERIALIZE_DATA) {
            $buff = unserialize($buff);
        }

        return $buff;
    }

    /**
     *
     * Appends a parameter to an url; uses '?' or '&'
     * It's the reverse of parse_str().
     *
     * @param string $url
     * @param array $params
     * @return string
     */
    public static function add_url_params($url, $params = array()) {
        $str = '';

        $params = (array) $params;

        if (empty($params)) {
            return $url;
        }

        $query_start = (strpos($url, '?') === false) ? '?' : '&';

        foreach ($params as $key => $value) {
            $str .= ( strlen($str) < 1) ? $query_start : '&';
            $str .= rawurlencode($key) . '=' . rawurlencode($value);
        }

        $str = $url . $str;

        return $str;
    }

    // generates HTML select
    public static function html_select($name = '', $options = array(), $sel = null, $attr = '') {
        $html = "\n" . '<select name="' . $name . '" ' . $attr . '>' . "\n";

        foreach ($options as $key => $label) {
            $selected = $sel == $key ? ' selected="selected"' : '';
            $html .= "\t<option value='$key' $selected>$label</option>\n";
        }

        $html .= '</select>';
        $html .= "\n";

        return $html;
    }

    // generates status msg
    public static function msg($msg = '', $status = 0) {
        $cls = empty($status) ? 'error' : 'success';
        $cls = $status == 2 ? 'notice' : $cls;

        $msg = "<p class='status_wrapper'><div class=\"status_msg $cls\">$msg</div></p>";

        return $msg;
    }

    /**
     * Adds missing namespaces because the like will not show up in IE 6,7,8 if they are not set
     * @param string $matched_str
     * @return string
     */
    public static function add_missing_namespaces($matched_str) {
        $og = 'xmlns:og="http://opengraphprotocol.org/schema/"';
        $fb = 'xmlns:fb="http://www.facebook.com/2008/fbml"';

        if (stripos($matched_str, 'xmlns:og') === false) {
            $matched_str .= ' ' . $og;
        }

        if (stripos($matched_str, 'xmlns:fb') === false) {
            $matched_str .= ' ' . $fb;
        }

        $matched_str = '<html' . stripslashes($matched_str) . '>';

        return $matched_str;
    }
}
